@@ -5,15 +5,16 @@ import time
 
 from typing import Union, List
 from datetime import datetime
+from assemblyline.common.uid import get_id_from_data
 
 import elasticapm
 import elasticsearch
-from assemblyline.datastore.exceptions import MultiKeyError
+from assemblyline.datastore.exceptions import MultiKeyError, VersionConflictException
 
 from assemblyline.common import forge
 from assemblyline.common.dict_utils import recursive_update, flatten
 from assemblyline.common.isotime import now_as_iso
-from assemblyline.datastore import Collection, log
+from assemblyline.datastore.collection import ESCollection, log
 from assemblyline.odm import Model, DATEFORMAT
 from assemblyline.odm.models.alert import Alert
 from assemblyline.odm.models.cached_file import CachedFile
@@ -34,7 +35,6 @@ from assemblyline.odm.models.user_favorites import UserFavorites
 from assemblyline.odm.models.user_settings import UserSettings
 from assemblyline.odm.models.safelist import Safelist
 from assemblyline.odm.models.workflow import Workflow
-from assemblyline.remote.datatypes.lock import Lock
 
 config = forge.get_config()
 
@@ -83,94 +83,90 @@ class AssemblylineDatastore(object):
         self.ds.archive_access = False
 
     @property
-    def alert(self) -> Collection:
+    def alert(self) -> ESCollection[Alert]:
         return self.ds.alert
 
     @property
-    def cached_file(self) -> Collection:
+    def cached_file(self) -> ESCollection[CachedFile]:
         return self.ds.cached_file
 
     @property
-    def emptyresult(self) -> Collection:
+    def emptyresult(self) -> ESCollection[EmptyResult]:
         return self.ds.emptyresult
 
     @property
-    def error(self) -> Collection:
+    def error(self) -> ESCollection[Error]:
         return self.ds.error
 
     @property
-    def file(self) -> Collection:
+    def file(self) -> ESCollection[File]:
         return self.ds.file
 
     @property
-    def filescore(self) -> Collection:
+    def filescore(self) -> ESCollection[FileScore]:
         return self.ds.filescore
 
     @property
-    def heuristic(self) -> Collection:
+    def heuristic(self) -> ESCollection[Heuristic]:
         return self.ds.heuristic
 
     @property
-    def result(self) -> Collection:
+    def result(self) -> ESCollection[Result]:
         return self.ds.result
 
     @property
-    def service(self) -> Collection:
+    def service(self) -> ESCollection[Service]:
         return self.ds.service
 
     @property
-    def service_client(self) -> Collection:
-        return self.ds.service_client
-
-    @property
-    def service_delta(self) -> Collection:
+    def service_delta(self) -> ESCollection[ServiceDelta]:
         return self.ds.service_delta
 
     @property
-    def signature(self) -> Collection:
+    def signature(self) -> ESCollection[Signature]:
         return self.ds.signature
 
     @property
-    def submission(self) -> Collection:
+    def submission(self) -> ESCollection[Submission]:
         return self.ds.submission
 
     @property
-    def submission_summary(self) -> Collection:
+    def submission_summary(self) -> ESCollection[SubmissionSummary]:
         return self.ds.submission_summary
 
     @property
-    def submission_tree(self) -> Collection:
+    def submission_tree(self) -> ESCollection[SubmissionTree]:
         return self.ds.submission_tree
 
     @property
-    def user(self) -> Collection:
+    def user(self) -> ESCollection[User]:
         return self.ds.user
 
     @property
-    def user_avatar(self) -> Collection:
+    def user_avatar(self) -> ESCollection:
         return self.ds.user_avatar
 
     @property
-    def user_favorites(self) -> Collection:
+    def user_favorites(self) -> ESCollection[UserFavorites]:
         return self.ds.user_favorites
 
     @property
-    def user_settings(self) -> Collection:
+    def user_settings(self) -> ESCollection[UserSettings]:
         return self.ds.user_settings
 
     @property
-    def vm(self) -> Collection:
+    def vm(self) -> ESCollection:
         return self.ds.vm
 
     @property
-    def safelist(self) -> Collection:
+    def safelist(self) -> ESCollection[Safelist]:
         return self.ds.safelist
 
     @property
-    def workflow(self) -> Collection:
+    def workflow(self) -> ESCollection[Workflow]:
         return self.ds.workflow
 
-    def get_collection(self, collection_name):
+    def get_collection(self, collection_name: str) -> ESCollection:
         if collection_name in self.ds.get_models():
             return getattr(self, collection_name)
         else:
@@ -420,15 +416,15 @@ class AssemblylineDatastore(object):
                             item.get('classification', cl_engine.UNRESTRICTED), new_file_class)
                         if item.get('classification', cl_engine.UNRESTRICTED) != new_class:
                             parts = cl_engine.get_access_control_parts(new_class)
-                            update_params = [(Collection.UPDATE_SET, 'classification', new_class)]
-                            update_params.extend([(Collection.UPDATE_SET, k, v) for k, v in parts.items()])
+                            update_params = [(ESCollection.UPDATE_SET, 'classification', new_class)]
+                            update_params.extend([(ESCollection.UPDATE_SET, k, v) for k, v in parts.items()])
                             self.result.update(item['id'], update_params)
 
                     # Alter the file classification if the new classification does not match
                     if cur_file['classification'] != new_file_class:
                         parts = cl_engine.get_access_control_parts(new_file_class)
-                        update_params = [(Collection.UPDATE_SET, 'classification', new_file_class)]
-                        update_params.extend([(Collection.UPDATE_SET, k, v) for k, v in parts.items()])
+                        update_params = [(ESCollection.UPDATE_SET, 'classification', new_file_class)]
+                        update_params.extend([(ESCollection.UPDATE_SET, k, v) for k, v in parts.items()])
                         self.file.update(f, update_params)
 
                     # Fix associated supplementary files
@@ -437,8 +433,8 @@ class AssemblylineDatastore(object):
                         if cur_supp:
                             if cur_supp['classification'] != new_file_class:
                                 parts = cl_engine.get_access_control_parts(new_file_class)
-                                update_params = [(Collection.UPDATE_SET, 'classification', new_file_class)]
-                                update_params.extend([(Collection.UPDATE_SET, k, v) for k, v in parts.items()])
+                                update_params = [(ESCollection.UPDATE_SET, 'classification', new_file_class)]
+                                update_params.extend([(ESCollection.UPDATE_SET, k, v) for k, v in parts.items()])
                                 self.file.update(supp, update_params)
 
         # Delete the submission and cached trees and summaries
@@ -536,11 +532,11 @@ class AssemblylineDatastore(object):
                                 user_classification=None):
         if user_classification is not None:
             user_classification = cl_engine.normalize_classification(user_classification, long_format=False)
-            cache_key = f"{submission['sid']}_{user_classification}"
+            cache_key = f"{submission['sid']}_{user_classification}_supp"
             for illegal_char in [" ", ":", "/"]:
                 cache_key = cache_key.replace(illegal_char, "")
         else:
-            cache_key = submission['sid']
+            cache_key = f"{submission['sid']}_supp"
 
         if isinstance(submission, Model):
             submission = submission.as_primitives()
@@ -556,13 +552,15 @@ class AssemblylineDatastore(object):
                     "tree": tree,
                     "classification": cached_tree['classification'],
                     "filtered": cached_tree['filtered'],
-                    "partial": False
+                    "partial": False,
+                    "supplementary": json.loads(cached_tree['supplementary'])
                 }
 
         partial = submission['state'] != 'completed'
         files = {}
         scores = {}
         missing_files = []
+        supplementary = []
         file_hashes = [x[:64] for x in submission['results']]
         file_hashes.extend([x[:64] for x in submission['errors']])
         file_hashes.extend([f['sha256'] for f in submission['files']])
@@ -609,6 +607,11 @@ class AssemblylineDatastore(object):
                 files[sha256] = []
             files[sha256].extend(extracted)
 
+            # Get supplementary files
+            if len(item['response']['supplementary']) == 0:
+                continue
+            supplementary.extend([x['sha256'] for x in item['response']['supplementary']])
+
         tree_cache = []
 
         def recurse_tree(child_p, placeholder, parents_p, lvl=0):
@@ -695,12 +698,16 @@ class AssemblylineDatastore(object):
                             "score": scores.get(sha256, 0),
                         }
 
+        # Cleanup supplementary
+        supplementary = list(set(supplementary))
+
         if not partial:
             cached_tree = {
                 'expiry_ts': now_as_iso(config.datastore.ilm.days_until_archive * 24 * 60 * 60),
                 'tree': json.dumps(tree),
                 'classification': max_classification,
-                'filtered': len(forbidden_files) > 0
+                'filtered': len(forbidden_files) > 0,
+                "supplementary": json.dumps(supplementary)
             }
 
             self.submission_tree.save(cache_key, cached_tree)
@@ -709,7 +716,8 @@ class AssemblylineDatastore(object):
             'tree': tree,
             'classification': max_classification,
             'filtered': len(forbidden_files) > 0,
-            'partial': partial
+            'partial': partial,
+            "supplementary": supplementary
         }
 
     @staticmethod
@@ -735,7 +743,8 @@ class AssemblylineDatastore(object):
         return True
 
     @elasticapm.capture_span(span_type='datastore')
-    def get_summary_from_keys(self, keys, cl_engine=forge.get_classification(), user_classification=None):
+    def get_summary_from_keys(self, keys, cl_engine=forge.get_classification(),
+                              user_classification=None, keep_heuristic_sections=False):
         out = {
             "tags": [],
             "attack_matrix": [],
@@ -746,12 +755,15 @@ class AssemblylineDatastore(object):
                 "malicious": []
             },
             "classification": cl_engine.UNRESTRICTED,
-            "filtered": False
+            "filtered": False,
+            "heuristic_sections": {},
+            "heuristic_name_map": {}
         }
         done_map = {
             "heuristics": set(),
             "attack": set(),
-            "tags": set()
+            "tags": set(),
+            "sections": set()
         }
 
         if len(keys) == 0:
@@ -775,7 +787,10 @@ class AssemblylineDatastore(object):
             out['missing_files'] = e.keys
 
         for key, item in items.items():
-            for section in item.get('result', {}).get('sections', []):
+            sorted_sections = sorted(item.get('result', {}).get('sections', []),
+                                     key=lambda i: i['heuristic']['score'] if i['heuristic'] is not None else 0,
+                                     reverse=True)
+            for section in sorted_sections:
                 file_classification = files.get(key[:64], {}).get('classification', section['classification'])
                 if user_classification:
                     if not cl_engine.is_accessible(user_classification, section['classification']):
@@ -791,25 +806,43 @@ class AssemblylineDatastore(object):
                 h_type = "info"
 
                 if section.get('heuristic', False):
+                    heur_id = section['heuristic']['heur_id']
+                    heur_name = section['heuristic']['name']
+
                     # Get the heuristics data
                     if section['heuristic']['score'] < 0:
                         h_type = "safe"
-                    elif section['heuristic']['score'] < 100:
+                    elif section['heuristic']['score'] < 300:
                         h_type = "info"
                     elif section['heuristic']['score'] < 1000:
                         h_type = "suspicious"
                     else:
                         h_type = "malicious"
 
-                    cache_key = f"{section['heuristic']['heur_id']}_{key}"
+                    cache_key = f"{heur_id}_{key}"
                     if cache_key not in done_map['heuristics']:
                         out['heuristics'][h_type].append({
-                            'heur_id': section['heuristic']['heur_id'],
-                            'name': section['heuristic']['name'],
+                            'heur_id': heur_id,
+                            'name': heur_name,
                             'key': key
                         })
                         done_map['heuristics'].add(cache_key)
 
+                    if keep_heuristic_sections:
+                        # Set defaults
+                        out['heuristic_sections'].setdefault(heur_id, [])
+                        out['heuristic_name_map'].setdefault(heur_name, [])
+
+                        # Set Name map
+                        if heur_id not in out['heuristic_name_map'][heur_name]:
+                            out['heuristic_name_map'][heur_name].append(heur_id)
+
+                        # Insert unique sections
+                        section_key = get_id_from_data(f"{heur_id}_{section['title_text']}_{section['body']}_{h_type}")
+                        if section_key not in done_map['sections']:
+                            out['heuristic_sections'][heur_id].append(section)
+                            done_map['sections'].add(section_key)
+
                     for attack in section['heuristic'].get('attack', []):
                         # Get attack matrix data
                         attack_id = attack['attack_id']
@@ -916,20 +949,21 @@ class AssemblylineDatastore(object):
         return out
 
     @elasticapm.capture_span(span_type='datastore')
-    def get_service_with_delta(self, service_name, version=None, as_obj=True):
+    def get_service_with_delta(self, service_name, version=None, as_obj=True) -> Union[Service, dict, None]:
         svc = self.ds.service_delta.get(service_name)
         if svc is None:
-            return svc
+            return None
 
         if version is not None:
             svc.version = version
 
         svc_version_data = self.ds.service.get(f"{service_name}_{svc.version}")
         if svc_version_data is None:
-            return svc_version_data
+            return None
 
         svc_version_data = recursive_update(svc_version_data.as_primitives(strip_null=True),
-                                            svc.as_primitives(strip_null=True))
+                                            svc.as_primitives(strip_null=True),
+                                            stop_keys=['config'])
         if as_obj:
             return Service(svc_version_data)
         else:
@@ -1018,24 +1052,23 @@ class AssemblylineDatastore(object):
         :param full: If true retrieve all the fields of the service object, otherwise only
                      fields returned by search are given.
         """
-        items = list(self.ds.service_delta.stream_search("id:*", as_obj=False))
+        mask = None if full else list(self.ds.service.stored_fields.keys())
 
-        if full:
-            service_data = self.ds.service.multiget([f"{item['id']}_{item['version']}" for item in items],
-                                                    as_dictionary=False)
-            service_delta = self.ds.service_delta.multiget([item['id'] for item in items], as_dictionary=False)
-            services = [recursive_update(data.as_primitives(strip_null=True), delta.as_primitives(strip_null=True))
-                        for data, delta in zip(service_data, service_delta)]
+        # List all services from service delta (Return all fields if full is true)
+        service_delta = list(self.ds.service_delta.stream_search("id:*", fl="*" if full else None))
 
-        else:
-            services_versions = {item['id']: item for item in self.ds.service.stream_search("id:*", as_obj=False)}
-            services = [recursive_update(services_versions[f"{item['id']}_{item['version']}"], item)
-                        for item in items if f"{item['id']}_{item['version']}" in services_versions]
+        # Gather all matching services and apply a mask if we don't want the full source object
+        service_data = [Service(s, mask=mask)
+                        for s in self.ds.service.multiget([f"{item.id}_{item.version}" for item in service_delta],
+                                                          as_obj=False, as_dictionary=False)]
 
+        # Recursively update the service data with the service delta while stripping nulls
+        services = [recursive_update(data.as_primitives(strip_null=True), delta.as_primitives(strip_null=True),
+                                     stop_keys=['config'])
+                    for data, delta in zip(service_data, service_delta)]
+
+        # Return as an objet if needs be...
         if as_obj:
-            mask = None
-            if not full and services:
-                mask = services[0].keys()
             return [Service(s, mask=mask) for s in services]
         else:
             return services
@@ -1050,21 +1083,47 @@ class AssemblylineDatastore(object):
 
     @elasticapm.capture_span(span_type='datastore')
     def save_or_freshen_file(self, sha256, fileinfo, expiry, classification,
-                             cl_engine=forge.get_classification(), redis=None):
-        with Lock(f'save-or-freshen-file-{sha256}', 5, host=redis):
-            current_fileinfo = self.ds.file.get_if_exists(
-                sha256, as_obj=False, force_archive_access=config.datastore.ilm.update_archive) or {}
+                             cl_engine=forge.get_classification(), redis=None, is_section_image=False):
+        # Remove control fields from new file info
+        for x in ['classification', 'expiry_ts', 'seen', 'archive_ts']:
+            fileinfo.pop(x, None)
+        # Clean up and prepare timestamps
+        if isinstance(expiry, datetime):
+            expiry = expiry.strftime(DATEFORMAT)
+        archive_time = now_as_iso(config.datastore.ilm.days_until_archive * 24 * 60 * 60)
 
-            # Remove control fields from file info and update current file info
-            for x in ['classification', 'expiry_ts', 'seen', 'archive_ts']:
-                fileinfo.pop(x, None)
+        while True:
+            current_fileinfo, version = self.ds.file.get_if_exists(
+                sha256, as_obj=False, archive_access=config.datastore.ilm.update_archive, version=True)
+
+            if current_fileinfo is None:
+                current_fileinfo = {}
+            else:
+                # If the freshen we are doing won't change classification, we can do it via an update operation
+                classification = cl_engine.min_classification(
+                    str(current_fileinfo.get('classification', classification)),
+                    str(classification)
+                )
+                if classification == current_fileinfo.get('classification', None):
+                    operations = [
+                        (self.ds.file.UPDATE_SET, key, value)
+                        for key, value in fileinfo.items()
+                    ]
+                    operations.extend([
+                        (self.ds.file.UPDATE_MAX, 'archive_ts', archive_time),
+                        (self.ds.file.UPDATE_INC, 'seen.count', 1),
+                        (self.ds.file.UPDATE_MAX, 'seen.last', now_as_iso()),
+                    ])
+                    if expiry:
+                        operations.append((self.ds.file.UPDATE_MAX, 'expiry_ts', expiry))
+                    if self.ds.file.update(sha256, operations):
+                        return
+
+            # Add new fileinfo to current from database
             current_fileinfo.update(fileinfo)
-
-            current_fileinfo['archive_ts'] = now_as_iso(config.datastore.ilm.days_until_archive * 24 * 60 * 60)
+            current_fileinfo['archive_ts'] = archive_time
 
             # Update expiry time
-            if isinstance(expiry, datetime):
-                expiry = expiry.strftime(DATEFORMAT)
             current_expiry = current_fileinfo.get('expiry_ts', expiry)
             if current_expiry and expiry:
                 current_fileinfo['expiry_ts'] = max(current_expiry, expiry)
@@ -1079,9 +1138,13 @@ class AssemblylineDatastore(object):
             seen['first'] = seen.get('first', now)
 
             # Update Classification
-            classification = cl_engine.min_classification(
-                str(current_fileinfo.get('classification', classification)),
-                str(classification)
-            )
             current_fileinfo['classification'] = classification
-            self.ds.file.save(sha256, current_fileinfo, force_archive_access=config.datastore.ilm.update_archive)
+
+            # Update section image status
+            current_fileinfo['is_section_image'] = current_fileinfo.get('is_section_image', False) or is_section_image
+
+            try:
+                self.ds.file.save(sha256, current_fileinfo, version=version)
+                return
+            except VersionConflictException as vce:
+                log.info(f"Retrying save or freshen due to version conflict: {str(vce)}")
