@@ -13,7 +13,7 @@ def normalize_srl_path(srl):
 class TransportException(ChainException):
     """
     FileTransport exception base class.
-    
+
     TransportException is a subclass of ChainException so that it can be
     used with the Chain and ChainAll decorators.
     """
@@ -23,7 +23,7 @@ class TransportException(ChainException):
 class Transport(object):
     """
     FileTransport base class.
-    
+
     - Subclasses should override all methods.
     - Except as noted, FileTransport methods do not return value and raise
     - TransportException on failure.
@@ -32,13 +32,13 @@ class Transport(object):
       respectively, to ensure that any exceptions raised are converted to
       TransportExceptions.
     """
-    
+
     def __init__(self, normalize=normalize_srl_path):
         self.normalize = normalize
 
     def close(self):
         pass
-    
+
     def delete(self, path: str):
         """
         Deletes the file.
@@ -72,26 +72,13 @@ class Transport(object):
         """
         raise TransportException("Not Implemented")
 
-    def upload_batch(self, local_remote_tuples):
-        """
-        Upload multiple files specified by list of (local, remote) tuples.
-        Transports that can optimize batch file transfers should write a custom upload_batch.
-        """
-        failed_tuples = []
-        for (src_path, dst_path) in local_remote_tuples:
-            try:
-                self.upload(src_path, dst_path)
-            except Exception as e:
-                failed_tuples.append((src_path, dst_path, str(e)))
-        return failed_tuples
-
     # Buffer based functions
     def get(self, path: str) -> bytes:
         """
         Returns the content of the file.
         """
         raise TransportException("Not Implemented")
-        
+
     def put(self, dst_path: str, content: AnyStr):
         """
         Put the content of the file in memory directly to the filestore dst_path
