@@ -4,6 +4,7 @@ import logging
 import os
 import posixpath
 import threading
+import socket
 import time
 import errno
 import weakref
@@ -57,6 +58,9 @@ def reconnect_retry_on_fail(func):
             except ftplib.Error as e:
                 msg = str(e) or "Unknown FTP Error"
 
+            except socket.gaierror as e:
+                msg = str(e) or "Unknown DNS Error"
+
             except IOError as e:
                 # May need to ignore other errors as well
                 if e.errno not in (errno.EPIPE, errno.ECONNRESET):
@@ -96,6 +100,7 @@ class TransportFTP(Transport):
     """
     FTP Transport class.
     """
+
     def __init__(self, base=None, host=None, password=None, user=None, port=None, use_tls=None):
         self.log: logging.Logger = logging.getLogger('assemblyline.transport.ftp')
         self.base: str = base
@@ -194,13 +199,9 @@ class TransportFTP(Transport):
         self.ftp.rename(temppath, finalpath)
         assert (self.exists(dst_path))
 
-    @reconnect_retry_on_fail
-    def upload_batch(self, local_remote_tuples):
-        return super(TransportFTP, self).upload_batch(local_remote_tuples)
-
     # Buffer based functions
     @reconnect_retry_on_fail
-    def get(self, path) -> bytes:
+    def get(self, path: str) -> bytes:
         path = self.normalize(path)
         bio = BytesIO()
         self.ftp.retrbinary('RETR ' + path, bio.write)
