@@ -489,14 +489,16 @@ class ScalerServiceDefaults(odm.Model):
 @odm.model(index=False, store=False)
 class Scaler(odm.Model):
     service_defaults: ScalerServiceDefaults = odm.Compound(ScalerServiceDefaults)
-    # only available for docker hosts, not kubernetes
-    cpu_overallocation: float = odm.Float(default=1)
-    memory_overallocation: float = odm.Float(default=1)
+    cpu_overallocation: float = odm.Float()
+    memory_overallocation: float = odm.Float()
     # Additional labels to be applied to deployments in kubernetes('=' delimited)
     additional_labels: List[str] = odm.Optional(odm.List(odm.Text()))
 
 
 DEFAULT_SCALER = {
+    'additional_labels': None,
+    'cpu_overallocation': 1,
+    'memory_overallocation': 1,
     'service_defaults': {
         'growth': 60,
         'shrink': 30,
@@ -703,10 +705,16 @@ class Services(odm.Model):
     stages: List[str] = odm.List(odm.Keyword())
     # Substitution variables for image paths (for custom registry support)
     image_variables: Dict[str, str] = odm.Mapping(odm.Keyword(default=''))
+    # Same as above, but only applied in the updater, used in dev setups and local registries
+    update_image_variables: Dict[str, str] = odm.Mapping(odm.Keyword(default=''))
     # Default update channel to be used for new services
     preferred_update_channel: str = odm.Keyword()
     # Allow container registries with self signed certs for service updates
     allow_insecure_registry: bool = odm.Boolean()
+    # Global registry type to be used for fetching updates for a service (can be overridable by a service)
+    preferred_registry_type: str = odm.Enum(values=["docker", "harbor"], default='docker')
+    # Global preference that controls if services should be privileged to communicate with core infrastucture
+    prefer_service_privileged:  bool = odm.Boolean(default=False)
     # How much CPU to reserve for services, at 1 a service's full cpu request will be reserved for them.
     # At 0 (only for very small appliances/dev boxes) the service's cpu will be limited
     # but no cpu will be reserved allowing for more flexible scheduling of containers.
@@ -720,6 +728,7 @@ DEFAULT_SERVICES = {
     "min_service_workers": 0,
     "stages": SERVICE_STAGES,
     "image_variables": {},
+    "update_image_variables": {},
     "preferred_update_channel": "stable",
     "allow_insecure_registry": False,
     "cpu_reservation": 0.25
@@ -771,13 +780,54 @@ DEFAULT_STATISTICS = {
 }
 
 
+# Option for the alerting perspective
+@odm.model(index=False, store=False)
+class AlertingMeta(odm.Model):
+    important: List[str] = odm.List(odm.Keyword())
+    subject: List[str] = odm.List(odm.Keyword())
+    url: List[str] = odm.List(odm.Keyword())
+
+
+DEFAULT_ALERTING_META = {
+    'important': [
+        'protocol',
+        'subject',
+        'submitted_url',
+        'source_url',
+        'url',
+        'web_url',
+        'from',
+        'to',
+        'cc',
+        'bcc',
+        'ip_src',
+        'ip_dst',
+        'source'
+    ],
+    'subject': [
+        'subject'
+    ],
+    'url': [
+        'submitted_url',
+        'source_url',
+        'url',
+        'web_url'
+    ]
+
+}
+
+
 # This is the model definition for the logging block
 @odm.model(index=False, store=False)
 class UI(odm.Model):
+    # Alerting metadata fields
+    alerting_meta: AlertingMeta = odm.Compound(AlertingMeta, default=DEFAULT_ALERTING_META)
     # Allow user to tell in advance the system that a file is malicious
     allow_malicious_hinting: bool = odm.Boolean()
-    # Allow to user to download raw files
+    # Allow user to download raw files
     allow_raw_downloads: bool = odm.Boolean()
+    # Allow users to request replay on another server
+    allow_replay: bool = odm.Boolean()
     # Allow file submissions via url
     allow_url_submissions: bool = odm.Boolean()
     # Should API calls be audited and saved to a separate log file?
@@ -788,6 +838,8 @@ class UI(odm.Model):
     banner_level: str = odm.Enum(values=["info", "warning", "success", "error"])
     # Turn on debugging
     debug: bool = odm.Boolean()
+    # Discover URL
+    discover_url: str = odm.Optional(odm.Keyword())
     # Which encoding will be used
     download_encoding = odm.Enum(values=["raw", "cart"])
     # Assemblyline admins email address
@@ -825,13 +877,16 @@ class UI(odm.Model):
 
 
 DEFAULT_UI = {
+    "alerting_meta": DEFAULT_ALERTING_META,
     "allow_malicious_hinting": False,
     "allow_raw_downloads": True,
+    "allow_replay": False,
     "allow_url_submissions": True,
     "audit": True,
     "banner": None,
     "banner_level": 'info',
     "debug": False,
+    "discover_url": None,
     "download_encoding": "cart",
     "email": None,
     "enforce_quota": True,
@@ -906,8 +961,10 @@ class Submission(odm.Model):
     max_extraction_depth: int = odm.Integer()
     # Maximum size for files submitted in the system
     max_file_size: int = odm.Integer()
-    # Maximum length for each metadata keys
+    # Maximum length for each metadata values
     max_metadata_length: int = odm.Integer()
+    # Maximum length for each temporary data values
+    max_temp_data_length: int = odm.Integer()
 
     # Summary tag types
     tag_types = odm.Compound(TagTypes, default=DEFAULT_TAG_TYPES)
@@ -921,6 +978,7 @@ DEFAULT_SUBMISSION = {
     'max_extraction_depth': 6,
     'max_file_size': 104857600,
     'max_metadata_length': 4096,
+    'max_temp_data_length': 4096,
     'tag_types': DEFAULT_TAG_TYPES
 }
 
@@ -965,5 +1023,5 @@ DEFAULT_CONFIG = {
 
 if __name__ == "__main__":
     # When executed, the config model will print the default values of the configuration
-    from pprint import pprint
-    pprint(DEFAULT_CONFIG)
+    import yaml
+    print(yaml.safe_dump(DEFAULT_CONFIG))
